package envoy.server.processors;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import envoy.data.Contacts;
import envoy.data.LoginCredentials;
import envoy.data.Message.MessageStatus;
import envoy.data.User;
import envoy.server.ConnectionManager;
import envoy.server.ObjectProcessor;
import envoy.server.data.Message;
import envoy.server.database.PersistenceManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * This {@link ObjectProcessor} handles {@link LoginCredentials}.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>LoginCredentialProcessor.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Server Standalone v0.1-alpha
 */
public class LoginCredentialProcessor implements ObjectProcessor<LoginCredentials> {

	// TODO: Acquire user IDs from database
	private static long currentUserId = 1;

	@Override
	public Class<LoginCredentials> getInputClass() { return LoginCredentials.class; }

	@Override
	public void process(LoginCredentials input, long socketId, ObjectWriteProxy writeProxy) throws IOException {
		System.out.println(String.format("Received login credentials %s from socket ID %d", input, socketId));

		// Create user
		User user = new User(currentUserId++, input.getName());
		ConnectionManager.getInstance().registerUser(socketId, user.getId());

		// Create contacts
		List<User>	users		= PersistenceManager.getPersistenceManager()
			.getContacts(new envoy.server.data.User(user))
			.stream()
			.map(envoy.server.data.User::toCommonUser)
			.collect(Collectors.toList());
		Contacts	contacts	= new Contacts(user.getId(), users);

		// Complete handshake
		System.out.println("Sending user...");
		writeProxy.write(socketId, user);
		System.out.println("Sending contacts...");
		writeProxy.write(socketId, contacts);
		System.out.println("Acquiring pending messages for the client...");
		List<Message> pendingMessages = PersistenceManager.getPersistenceManager().getUnreadMessages(new envoy.server.data.User(user));
		for (Message msg : pendingMessages) {
			System.out.println("Sending message " + msg.toString());
			writeProxy.write(socketId, msg);
			msg.setReceivedDate(new Date());
			msg.setStatus(MessageStatus.RECEIVED);
			PersistenceManager.getPersistenceManager().updateMessage(msg);
		}
	}
}
